@@ -11,23 +11,19 @@ PARSER_SRC_DIR="$SCRIPTS_DIR/../src/scriptvm/editor"
 
 echo -n "Searching for lexer..."
 
-LEX_CMD=NONE
+FLEX_CMD=NONE
 if which "flex" > /dev/null; then
-    LEX_CMD=`which flex`
-elif which "lex" > /dev/null; then
-    LEX_CMD=`which lex`
+    FLEX_CMD=`which flex`
 else
-    echo "Error: You need lex (or flex) to generate the NKSP syntax highlighting scanner !"
+    echo "Error: You need flex to generate the NKSP syntax highlighting scanner !"
     exit -1
 fi
 
-echo "OK ($LEX_CMD)"
+echo "OK ($FLEX_CMD)"
 
 echo -n "Generating NKSP editor syntax highlighting scanner ... "
 (
     cd $PARSER_SRC_DIR
-
-    $LEX_CMD nksp.l
-    mv -f lex.Nksp_.c nksp.cpp 2>/dev/null
+    $FLEX_CMD -o nksp.cpp nksp.l
 )
 echo "Done"
