@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005, 2006 Christian Schoenebeck                        *
+ *   Copyright (C) 2005 - 2025 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -26,6 +26,8 @@
 
 #include "DeviceParameter.h"
 
+#include "../common/global_private.h"
+
 namespace LinuxSampler {
 
     // if string is encapsulated into apostrophes or quotation marks, then remove those apostrophes / quotation marks
@@ -39,8 +41,18 @@ namespace LinuxSampler {
         }
     }
 
-    static bool __parse_bool(String val) throw (Exception) {
-        __eliminate_quotation(val);
+    // like __eliminate_quotation(), but this one really kills all of 'em
+    static void __eliminate_all_quotations(String& s) {
+        for (int i = 0; i < s.size(); i++) {
+            if (s.c_str()[i] == '\'' || s.c_str()[i] == '\"') {
+                s.replace(i, 1, "");
+                i--;
+            }
+        }
+    }
+
+    static bool __parse_bool(String val) LS_THROWS(Exception) {
+        __eliminate_all_quotations(val);
         int b;
         if      (val == "1" || !strcasecmp(val.c_str(),"true"))  b = true;
         else if (val == "0" || !strcasecmp(val.c_str(),"false")) b = false;
@@ -48,24 +60,31 @@ namespace LinuxSampler {
         return b;
     }
 
-    static int __parse_int(String val) throw (Exception) {
-        __eliminate_quotation(val);
+    static int __parse_int(String val) LS_THROWS(Exception) {
+        __eliminate_all_quotations(val);
         return atoi(val.c_str()); // TODO: format check is missing
     }
 
-    static float __parse_float(String val) throw (Exception) {
-        __eliminate_quotation(val);
-        return atof(val.c_str()); // TODO: format check is missing
+    static float __parse_float(String val) LS_THROWS(Exception) {
+        __eliminate_all_quotations(val);
+        float x;
+        std::stringstream ss(val);
+        ss.imbue(std::locale::classic());
+        ss >> x; // TODO: format check is missing
+        return x;
     }
 
-    static String __parse_string(String val) {        
+    static String __parse_string(String val) {
         __eliminate_quotation(val);
         return val;
     }
 
-    static std::vector<String> __parse_strings(String val) throw (Exception) {
+    static std::vector<String> __parse_strings(String val) LS_THROWS(Exception) {
         std::vector<String> vS;
 
+        // checking for empty list
+        if (val.length() == 0) return vS;
+
         // if there's only a single value, then we also allow to give it without being encapsulated into apostrophes
         if (val.find("\'") == String::npos && val.find("\"") == String::npos) {
             vS.push_back(val);
@@ -130,7 +149,7 @@ namespace LinuxSampler {
         return (ValueAsBool()) ? "true" : "false";
     }
 
-    void DeviceRuntimeParameterBool::SetValue(String val) throw (Exception) {
+    void DeviceRuntimeParameterBool::SetValue(String val) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         int b = __parse_bool(val);
         SetValue(b);
@@ -140,7 +159,7 @@ namespace LinuxSampler {
         return bVal;
     }
 
-    void DeviceRuntimeParameterBool::SetValue(bool b) throw (Exception) {
+    void DeviceRuntimeParameterBool::SetValue(bool b) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         OnSetValue(b);
         bVal = b;
@@ -193,7 +212,7 @@ namespace LinuxSampler {
         return ToString(ValueAsInt());
     }
 
-    void DeviceRuntimeParameterInt::SetValue(String val) throw (Exception) {
+    void DeviceRuntimeParameterInt::SetValue(String val) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         int i = __parse_int(val);
         if (RangeMinAsInt() && i < *RangeMinAsInt()) throw Exception("Invalid device parameter value: too small");
@@ -219,7 +238,7 @@ namespace LinuxSampler {
         return iVal;
     }
 
-    void DeviceRuntimeParameterInt::SetValue(int i) throw (Exception) {
+    void DeviceRuntimeParameterInt::SetValue(int i) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         OnSetValue(i);
         iVal = i;
@@ -272,7 +291,7 @@ namespace LinuxSampler {
         return ToString(ValueAsFloat());
     }
 
-    void DeviceRuntimeParameterFloat::SetValue(String val) throw (Exception) {
+    void DeviceRuntimeParameterFloat::SetValue(String val) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         float f = __parse_float(val);
         if (RangeMinAsFloat() && f < *RangeMinAsFloat()) throw Exception("Invalid device parameter value: too small");
@@ -298,7 +317,7 @@ namespace LinuxSampler {
         return fVal;
     }
 
-    void DeviceRuntimeParameterFloat::SetValue(float f) throw (Exception) {
+    void DeviceRuntimeParameterFloat::SetValue(float f) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         OnSetValue(f);
         fVal = f;
@@ -347,7 +366,7 @@ namespace LinuxSampler {
         return "\'" + ValueAsString() + "\'";
     }
 
-    void DeviceRuntimeParameterString::SetValue(String val) throw (Exception) {
+    void DeviceRuntimeParameterString::SetValue(String val) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         SetValueAsString(__parse_string(val));
     }
@@ -356,7 +375,7 @@ namespace LinuxSampler {
         return sVal;
     }
 
-    void DeviceRuntimeParameterString::SetValueAsString(String val) throw (Exception) {
+    void DeviceRuntimeParameterString::SetValueAsString(String val) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         if (val.find("\'") != String::npos) throw Exception("Character -> \' <- not allowed");
         if (val.find("\"") != String::npos) throw Exception("Character -> \" <- not allowed");
@@ -414,7 +433,7 @@ namespace LinuxSampler {
         return result;
     }
 
-    void DeviceRuntimeParameterStrings::SetValue(String val) throw (Exception) {
+    void DeviceRuntimeParameterStrings::SetValue(String val) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         std::vector<String> vS = __parse_strings(val);
         SetValue(vS);
@@ -424,7 +443,7 @@ namespace LinuxSampler {
         return sVals;
     }
 
-    void DeviceRuntimeParameterStrings::SetValue(std::vector<String> vS) throw (Exception) {
+    void DeviceRuntimeParameterStrings::SetValue(std::vector<String> vS) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         OnSetValue(vS);
         sVals = vS;
@@ -480,7 +499,7 @@ namespace LinuxSampler {
         this->bVal = bVal;
     }
 
-    DeviceCreationParameterBool::DeviceCreationParameterBool(String val) throw (Exception) {
+    DeviceCreationParameterBool::DeviceCreationParameterBool(String val) LS_THROWS(Exception) {
         this->bVal = __parse_bool(val);
     }
 
@@ -520,7 +539,7 @@ namespace LinuxSampler {
         return (ValueAsBool()) ? "true" : "false";
     }
 
-    void DeviceCreationParameterBool::SetValue(String val) throw (Exception) {
+    void DeviceCreationParameterBool::SetValue(String val) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         int b = __parse_bool(val);
         SetValue(b);
@@ -530,7 +549,7 @@ namespace LinuxSampler {
         return bVal;
     }
 
-    void DeviceCreationParameterBool::SetValue(bool b) throw (Exception) {
+    void DeviceCreationParameterBool::SetValue(bool b) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         OnSetValue(b);
         bVal = b;
@@ -545,7 +564,7 @@ namespace LinuxSampler {
         this->iVal = iVal;
     }
 
-    DeviceCreationParameterInt::DeviceCreationParameterInt(String val) throw (Exception) {
+    DeviceCreationParameterInt::DeviceCreationParameterInt(String val) LS_THROWS(Exception) {
         this->iVal = __parse_int(val);
     }
 
@@ -599,16 +618,23 @@ namespace LinuxSampler {
         return ToString(ValueAsInt());
     }
 
-    void DeviceCreationParameterInt::SetValue(String val) throw (Exception) {
+    void DeviceCreationParameterInt::SetValue(String val) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         int i = __parse_int(val);
-        //if (RangeMinAsInt() && i < *RangeMinAsInt()) throw Exception("Invalid device parameter value: too small");
-        //if (RangeMaxAsInt() && i > *RangeMaxAsInt()) throw Exception("Invalid device parameter value: too big");
-        /*if (PossibilitiesAsInt()) {
+
+        std::map<String,String> emptyMap;
+        if (RangeMinAsInt(emptyMap) && i < *RangeMinAsInt(emptyMap)) {
+            throw Exception("Invalid device parameter value: too small");
+        }
+        if (RangeMaxAsInt(emptyMap) && i > *RangeMaxAsInt(emptyMap)) {
+            throw Exception("Invalid device parameter value: too big");
+        }
+
+        if (PossibilitiesAsInt(emptyMap).size()) {
             bool valid = false;
-            std::vector<int>* pPossibilities = PossibilitiesAsInt();
-            std::vector<int>::iterator iter  = pPossibilities->begin();
-            while (iter != pPossibilities->end()) {
+            std::vector<int> possibilities = PossibilitiesAsInt(emptyMap);
+            std::vector<int>::iterator iter  = possibilities.begin();
+            while (iter != possibilities.end()) {
                 if (i == *iter) {
                     valid = true;
                     break;
@@ -616,7 +642,7 @@ namespace LinuxSampler {
                 iter++;
             }
             if (!valid) throw Exception("Invalid Device parameter value: not in set of possible values");
-        }*/
+        }
         SetValue(i);
     }
 
@@ -624,7 +650,7 @@ namespace LinuxSampler {
         return iVal;
     }
 
-    void DeviceCreationParameterInt::SetValue(int i) throw (Exception) {
+    void DeviceCreationParameterInt::SetValue(int i) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         OnSetValue(i);
         iVal = i;
@@ -639,7 +665,7 @@ namespace LinuxSampler {
         this->fVal = fVal;
     }
 
-    DeviceCreationParameterFloat::DeviceCreationParameterFloat(String val) throw (Exception) {
+    DeviceCreationParameterFloat::DeviceCreationParameterFloat(String val) LS_THROWS(Exception) {
         this->fVal = __parse_float(val);
     }
 
@@ -693,7 +719,7 @@ namespace LinuxSampler {
         return ToString(ValueAsFloat());
     }
 
-    void DeviceCreationParameterFloat::SetValue(String val) throw (Exception) {
+    void DeviceCreationParameterFloat::SetValue(String val) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         float f = __parse_float(val);
         //if (RangeMinAsFloat() && i < *RangeMinAsFloat()) throw Exception("Invalid device parameter value: too small");
@@ -718,7 +744,7 @@ namespace LinuxSampler {
         return fVal;
     }
 
-    void DeviceCreationParameterFloat::SetValue(float f) throw (Exception) {
+    void DeviceCreationParameterFloat::SetValue(float f) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         OnSetValue(f);
         fVal = f;
@@ -780,7 +806,7 @@ namespace LinuxSampler {
         return "\'" + ValueAsString() + "\'";
     }
 
-    void DeviceCreationParameterString::SetValue(String val) throw (Exception) {
+    void DeviceCreationParameterString::SetValue(String val) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         SetValueAsString(__parse_string(val));
     }
@@ -789,7 +815,7 @@ namespace LinuxSampler {
         return sVal;
     }
 
-    void DeviceCreationParameterString::SetValueAsString(String val) throw (Exception) {
+    void DeviceCreationParameterString::SetValueAsString(String val) LS_THROWS(Exception) {
         if (val.find("\'") != String::npos) throw Exception("Character -> \' <- not allowed");
         if (val.find("\"") != String::npos) throw Exception("Character -> \" <- not allowed");
         OnSetValue(val);
@@ -805,7 +831,7 @@ namespace LinuxSampler {
         this->sVals = sVals;
     }
 
-    DeviceCreationParameterStrings::DeviceCreationParameterStrings(String val) throw (Exception) {
+    DeviceCreationParameterStrings::DeviceCreationParameterStrings(String val) LS_THROWS(Exception) {
         this->sVals = __parse_strings(val);
     }
 
@@ -868,7 +894,7 @@ namespace LinuxSampler {
         return result;
     }
 
-    void DeviceCreationParameterStrings::SetValue(String val) throw (Exception) {
+    void DeviceCreationParameterStrings::SetValue(String val) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         std::vector<String> vS = __parse_strings(val);
         SetValue(vS);
@@ -878,7 +904,7 @@ namespace LinuxSampler {
         return sVals;
     }
 
-    void DeviceCreationParameterStrings::SetValue(std::vector<String> vS) throw (Exception) {
+    void DeviceCreationParameterStrings::SetValue(std::vector<String> vS) LS_THROWS(Exception) {
         if (Fix()) throw Exception("Device parameter is read only");
         OnSetValue(vS);
         sVals = vS;
