@@ -1,6 +1,6 @@
 /***************************************************************************
  *                                                                         *
- *   Copyright (C) 2006 - 2020 Christian Schoenebeck                       *
+ *   Copyright (C) 2006 - 2025 Christian Schoenebeck                       *
  *                                                                         *
  *   This library is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -38,7 +38,7 @@ namespace LinuxSampler {
         InitWithDefault(); // use default name
     }
 
-    AudioOutputDeviceArts::ParameterName::ParameterName(String s) throw (Exception) : DeviceCreationParameterString(s) {
+    AudioOutputDeviceArts::ParameterName::ParameterName(String s) LS_THROWS(Exception) : DeviceCreationParameterString(s) {
     }
 
     String AudioOutputDeviceArts::ParameterName::Description() {
@@ -65,7 +65,7 @@ namespace LinuxSampler {
         return (existingArtsDevices) ? "LinuxSampler" + ToString(existingArtsDevices) : "LinuxSampler";
     }
 
-    void AudioOutputDeviceArts::ParameterName::OnSetValue(String s) throw (Exception) {
+    void AudioOutputDeviceArts::ParameterName::OnSetValue(String s) LS_THROWS(Exception) {
         // not possible, as parameter is fix
     }
 
@@ -161,7 +161,7 @@ namespace LinuxSampler {
     }
 
     String AudioOutputDeviceArts::Version() {
-       String s = "$Revision: 3766 $";
+       String s = "$Revision: 4167 $";
        return s.substr(11, s.size() - 13); // cut dollar signs, spaces and CVS macro keyword
     }
 
