@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005 - 2014 Christian Schoenebeck                       *
+ *   Copyright (C) 2005 - 2025 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -574,7 +574,7 @@ AudioOutputDeviceAsio::ParameterCard::ParameterCard() : DeviceCreationParameterS
     InitWithDefault(); // use default card
 }
 
-AudioOutputDeviceAsio::ParameterCard::ParameterCard(String s) throw (Exception) : DeviceCreationParameterString(s) {
+AudioOutputDeviceAsio::ParameterCard::ParameterCard(String s) LS_THROWS(Exception) : DeviceCreationParameterString(s) {
 }
 
 String AudioOutputDeviceAsio::ParameterCard::Description() {
@@ -624,7 +624,7 @@ std::vector<String> AudioOutputDeviceAsio::ParameterCard::PossibilitiesAsString(
     return getAsioDriverNames();
 }
 
-void AudioOutputDeviceAsio::ParameterCard::OnSetValue(String s) throw (Exception) {
+void AudioOutputDeviceAsio::ParameterCard::OnSetValue(String s) LS_THROWS(Exception) {
     // not posssible, as parameter is fix
 }
 
@@ -760,7 +760,7 @@ AudioOutputDeviceAsio::ParameterFragmentSize::ParameterFragmentSize() : DeviceCr
     InitWithDefault();
 }
 
-AudioOutputDeviceAsio::ParameterFragmentSize::ParameterFragmentSize(String s) throw (Exception) : DeviceCreationParameterInt(s) {
+AudioOutputDeviceAsio::ParameterFragmentSize::ParameterFragmentSize(String s) LS_THROWS(Exception) : DeviceCreationParameterInt(s) {
 }
 
 String AudioOutputDeviceAsio::ParameterFragmentSize::Description() {
@@ -829,7 +829,7 @@ std::vector<int> AudioOutputDeviceAsio::ParameterFragmentSize::PossibilitiesAsIn
     return std::vector<int>();
 }
 
-void AudioOutputDeviceAsio::ParameterFragmentSize::OnSetValue(int i) throw (Exception) {
+void AudioOutputDeviceAsio::ParameterFragmentSize::OnSetValue(int i) LS_THROWS(Exception) {
     // not posssible, as parameter is fix
 }
 
@@ -973,7 +973,7 @@ String AudioOutputDeviceAsio::Description() {
 }
 
 String AudioOutputDeviceAsio::Version() {
-    String s = "$Revision: 2509 $";
+    String s = "$Revision: 4167 $";
     return s.substr(11, s.size() - 13); // cut dollar signs, spaces and CVS macro keyword
 }
 
