@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005 - 2013 Christian Schoenebeck                       *
+ *   Copyright (C) 2005 - 2025 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -68,14 +68,14 @@ namespace LinuxSampler { //
             class ParameterCard : public DeviceCreationParameterString {
                 public:
                     ParameterCard();
-                    ParameterCard(String s) throw (Exception);
+                    ParameterCard(String s) LS_THROWS(Exception);
                     virtual String Description() OVERRIDE;
                     virtual bool   Fix() OVERRIDE;
                     virtual bool   Mandatory() OVERRIDE;
                     virtual std::map<String,DeviceCreationParameter*> DependsAsParameters() OVERRIDE;
                     virtual optional<String>    DefaultAsString(std::map<String,String> Parameters) OVERRIDE;
                     virtual std::vector<String> PossibilitiesAsString(std::map<String,String> Parameters) OVERRIDE;
-                    virtual void                OnSetValue(String s) throw (Exception) OVERRIDE;
+                    virtual void                OnSetValue(String s) LS_THROWS(Exception) OVERRIDE;
                     static String Name();
             };
 
@@ -115,7 +115,7 @@ namespace LinuxSampler { //
             class ParameterFragmentSize : public DeviceCreationParameterInt {
                 public:
                     ParameterFragmentSize();
-                    ParameterFragmentSize(String s) throw (Exception);
+                    ParameterFragmentSize(String s) LS_THROWS(Exception);
                     virtual String Description() OVERRIDE;
                     virtual bool   Fix() OVERRIDE;
                     virtual bool   Mandatory() OVERRIDE;
@@ -124,7 +124,7 @@ namespace LinuxSampler { //
                     virtual optional<int>    RangeMinAsInt(std::map<String,String> Parameters) OVERRIDE;
                     virtual optional<int>    RangeMaxAsInt(std::map<String,String> Parameters) OVERRIDE;
                     virtual std::vector<int> PossibilitiesAsInt(std::map<String,String> Parameters) OVERRIDE;
-                    virtual void             OnSetValue(int i) throw (Exception) OVERRIDE;
+                    virtual void             OnSetValue(int i) LS_THROWS(Exception) OVERRIDE;
                     static String Name();
 
             };
