@@ -4,7 +4,7 @@
  *                                                                         *
  *   Copyright (C) 2009 Grigor Iliev                                       *
  *   Copyright (C) 2011-2013 Andreas Persson                               *
- *   Copyright (C) 2014-2020 Christian Schoenebeck                         *
+ *   Copyright (C) 2014-2025 Christian Schoenebeck                         *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -165,7 +165,7 @@ namespace LinuxSampler {
     }
 
     String AudioOutputDeviceCoreAudio::Version() {
-       String s = "$Revision: 3766 $";
+       String s = "$Revision: 4167 $";
        return s.substr(11, s.size() - 13); // cut dollar signs, spaces and CVS macro keyword
     }
 
@@ -216,7 +216,7 @@ namespace LinuxSampler {
         }
     }
 
-    void AudioOutputDeviceCoreAudio::CreateAndStartAudioQueue() throw(Exception) {
+    void AudioOutputDeviceCoreAudio::CreateAndStartAudioQueue() LS_THROWS(Exception) {
         OSStatus res = AudioQueueNewOutput (
             &aqPlayerState.mDataFormat,
             HandleOutputBuffer,
@@ -370,7 +370,7 @@ namespace LinuxSampler {
     }
 
     AudioOutputDeviceCoreAudio::ParameterDevice::ParameterDevice(String s)
-            throw (Exception) : DeviceCreationParameterString(s) {
+            LS_THROWS(Exception) : DeviceCreationParameterString(s) {
     }
 
     String AudioOutputDeviceCoreAudio::ParameterDevice::Description() {
@@ -421,7 +421,7 @@ namespace LinuxSampler {
         return ToString(devIndex + 1) + " " + devs->GetOutputDevice(devIndex).GetName();
     }
 
-    void AudioOutputDeviceCoreAudio::ParameterDevice::OnSetValue(String s) throw (Exception) {
+    void AudioOutputDeviceCoreAudio::ParameterDevice::OnSetValue(String s) LS_THROWS(Exception) {
         // not posssible, as parameter is fix
     }
 
@@ -541,7 +541,7 @@ namespace LinuxSampler {
     }
 
     AudioOutputDeviceCoreAudio::ParameterBuffers::ParameterBuffers(String s)
-            throw (Exception) : DeviceCreationParameterInt(s) {
+            LS_THROWS(Exception) : DeviceCreationParameterInt(s) {
     }
 
     String AudioOutputDeviceCoreAudio::ParameterBuffers::Description() {
@@ -581,7 +581,7 @@ namespace LinuxSampler {
         return std::vector<int>();
     }
 
-    void AudioOutputDeviceCoreAudio::ParameterBuffers::OnSetValue(int i) throw (Exception) {
+    void AudioOutputDeviceCoreAudio::ParameterBuffers::OnSetValue(int i) LS_THROWS(Exception) {
         // not posssible, as parameter is fix
     }
 
@@ -597,7 +597,7 @@ namespace LinuxSampler {
     }
 
     AudioOutputDeviceCoreAudio::ParameterBufferSize::ParameterBufferSize(String s)
-            throw (Exception) : DeviceCreationParameterInt(s) {
+            LS_THROWS(Exception) : DeviceCreationParameterInt(s) {
     }
 
     String AudioOutputDeviceCoreAudio::ParameterBufferSize::Description() {
@@ -637,7 +637,7 @@ namespace LinuxSampler {
         return std::vector<int>();
     }
 
-    void AudioOutputDeviceCoreAudio::ParameterBufferSize::OnSetValue(int i) throw (Exception) {
+    void AudioOutputDeviceCoreAudio::ParameterBufferSize::OnSetValue(int i) LS_THROWS(Exception) {
         // not posssible, as parameter is fix
     }
 
