@@ -1,7 +1,7 @@
 /***************************************************************************
  *                                                                         *
  *   Copyright (C) 2004, 2005 Grame                                        *
- *   Copyright (C) 2005 - 2016 Christian Schoenebeck                       *
+ *   Copyright (C) 2005 - 2025 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -47,8 +47,8 @@ namespace LinuxSampler {
                      */
                     class ParameterName : public MidiInputPort::ParameterName {
                         public:
-                            ParameterName(MidiInputPort* pPort) throw (Exception);
-                            virtual void OnSetValue(String s) throw (Exception) OVERRIDE;
+                            ParameterName(MidiInputPort* pPort) LS_THROWS(Exception);
+                            virtual void OnSetValue(String s) LS_THROWS(Exception) OVERRIDE;
                     };
 
                     /** MIDI Port Parameter 'CORE_MIDI_BINDINGS'
@@ -62,7 +62,7 @@ namespace LinuxSampler {
                             virtual String Description() OVERRIDE;
                             virtual bool Fix() OVERRIDE;
                             virtual std::vector<String> PossibilitiesAsString() OVERRIDE;
-                            virtual void OnSetValue(std::vector<String> vS) throw (Exception) OVERRIDE;
+                            virtual void OnSetValue(std::vector<String> vS) LS_THROWS(Exception) OVERRIDE;
                         protected:
                             MidiInputPortCoreMidi* pPort;
                     };
@@ -77,7 +77,7 @@ namespace LinuxSampler {
 							ParameterAutoBind(MidiInputPortCoreMidi* pPort);
 							virtual String Description() OVERRIDE;
 							virtual bool Fix() OVERRIDE;
-							virtual void OnSetValue(bool b) throw (Exception) OVERRIDE;
+							virtual void OnSetValue(bool b) LS_THROWS(Exception) OVERRIDE;
 						protected:
 							MidiInputPortCoreMidi* pPort;
 					};
@@ -88,7 +88,7 @@ namespace LinuxSampler {
 					static int pPortID;
 
                 protected:
-                    MidiInputPortCoreMidi(MidiInputDeviceCoreMidi* pDevice) throw (MidiInputException);
+                    MidiInputPortCoreMidi(MidiInputDeviceCoreMidi* pDevice) LS_THROWS(MidiInputException);
                     ~MidiInputPortCoreMidi();
 					void connectToSource(MIDIEndpointRef source);
 					void connectToAllSources();
