@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005 - 2013 Christian Schoenebeck                       *
+ *   Copyright (C) 2005 - 2025 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -54,7 +54,7 @@ namespace LinuxSampler {
                     virtual optional<int>    RangeMinAsInt(std::map<String,String> Parameters) OVERRIDE;
                     virtual optional<int>    RangeMaxAsInt(std::map<String,String> Parameters) OVERRIDE;
                     virtual std::vector<int> PossibilitiesAsInt(std::map<String,String> Parameters) OVERRIDE;
-                    virtual void             OnSetValue(int i) throw (Exception) OVERRIDE;
+                    virtual void             OnSetValue(int i) LS_THROWS(Exception) OVERRIDE;
                     static String Name();
             };
 
@@ -82,7 +82,7 @@ namespace LinuxSampler {
                     };
                     
                 protected:
-                    MidiInputPortMme(MidiInputDeviceMme* pDevice) throw (MidiInputException);
+                    MidiInputPortMme(MidiInputDeviceMme* pDevice) LS_THROWS(MidiInputException);
                     ~MidiInputPortMme();
                     void ConnectToMmeMidiSource(const char* MidiSource);
                     void CloseMmeMidiPort(void);
