@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005 - 2009 Christian Schoenebeck                       *
+ *   Copyright (C) 2005 - 2025 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -54,10 +54,10 @@ namespace LinuxSampler {
                 public:
                     ParameterName(MidiInputPort* pPort);
                     ParameterName(MidiInputPort* pPort, String val);
-                    virtual String Description();
-                    virtual bool   Fix();
-                    virtual std::vector<String> PossibilitiesAsString();
-                    virtual void OnSetValue(String s) throw (Exception);
+                    virtual String Description() OVERRIDE;
+                    virtual bool   Fix() OVERRIDE;
+                    virtual std::vector<String> PossibilitiesAsString() OVERRIDE;
+                    virtual void OnSetValue(String s) LS_THROWS(Exception) OVERRIDE;
                 protected:
                     MidiInputPort* pPort;
             };
@@ -133,6 +133,29 @@ namespace LinuxSampler {
              * Disconnect the previously connected virtual MIDI device.
              */
             void Disconnect(VirtualMidiDevice* pDevice);
+            
+            /**
+             * Registers the given @a filter to be applied against all note on
+             * events, adjusting their MIDI velocity data. The vector supplied
+             * here must exactly be either of size 128 or 0. In case the given
+             * vector is of size 128, it will be used as lookup table by this
+             * MIDI inpurt port to remap any incoming MIDI note on velocity
+             * data to the respective value in that vector, or exactly:
+             * @code
+             * velocity = filter[velocity];
+             * @endcode
+             * Accordingly the values in the vector have to be in the range
+             * 0..127, however note that a value 0 should actually never be
+             * used, since by MIDI specification, a note on with velocity 0 is
+             * interpreted as note off event instead!
+             *
+             * If a vector of size 0 is supplied, this note on velocity filter
+             * mechanism will be disabled.
+             *
+             * @param filter - lookup table in the format described above
+             * @throws MidiInputException - if filter is in invalid format
+             */
+            void SetNoteOnVelocityFilter(const std::vector<uint8_t>& filter);
 
 
             /////////////////////////////////////////////////////////////////
@@ -275,6 +298,78 @@ namespace LinuxSampler {
              */
             void DispatchControlChange(uint8_t Controller, uint8_t Value, uint MidiChannel, int32_t FragmentPos);
 
+            /**
+             * Should be called by the implementing MIDI input device whenever
+             * a channel pressure event arrived (a.k.a. aftertouch), this will
+             * cause the channel pressure event to be forwarded to all engines
+             * on the corresponding MIDI channel.
+             *
+             * This method is meant for realtime rendering, this way an event
+             * is immediately created with the current system time as time
+             * stamp.
+             *
+             * @param Value       - MIDI channel pressure value (0..127)
+             * @param MidiChannel - MIDI channel on which event occured on
+             *                      (low level indexing, means 0..15)
+             * @see DispatchPolyphonicKeyPressure()
+             */
+            void DispatchChannelPressure(uint8_t Value, uint MidiChannel);
+
+            /**
+             * Should be called by the implementing MIDI input device whenever
+             * a channel pressure event arrived (a.k.a. aftertouch), this will
+             * cause the channel pressure event to be forwarded to all engines
+             * on the corresponding MIDI channel.
+             *
+             * This method is meant for offline rendering and / or in case the
+             * exact fragment position of the event is already known.
+             *
+             * @param Value       - MIDI channel pressure value (0..127)
+             * @param MidiChannel - MIDI channel on which event occured on
+             *                      (low level indexing, means 0..15)
+             * @param FragmentPos - event's sample point position in the
+             *                      current audio fragment
+             * @see DispatchPolyphonicKeyPressure()
+             */
+            void DispatchChannelPressure(uint8_t Value, uint MidiChannel, int32_t FragmentPos);
+
+            /**
+             * Should be called by the implementing MIDI input device whenever
+             * a polyphonic key pressure event arrived (a.k.a. polyphonic
+             * aftertouch), this will cause the polyphonic key pressure event
+             * to be forwarded to all engines on the corresponding MIDI channel.
+             *
+             * This method is meant for realtime rendering, this way an event
+             * is immediately created with the current system time as time
+             * stamp.
+             *
+             * @param Key         - MIDI key number of the key where pressure changed
+             * @param Value       - MIDI key pressure value (0..127)
+             * @param MidiChannel - MIDI channel on which event occured on
+             *                      (low level indexing, means 0..15)
+             * @see DispatchChannelPressure()
+             */
+            void DispatchPolyphonicKeyPressure(uint8_t Key, uint8_t Value, uint MidiChannel);
+
+            /**
+             * Should be called by the implementing MIDI input device whenever
+             * a polyphonic key pressure event arrived (a.k.a. polyphonic
+             * aftertouch), this will cause the polyphonic key pressure event
+             * to be forwarded to all engines on the corresponding MIDI channel.
+             *
+             * This method is meant for offline rendering and / or in case the
+             * exact fragment position of the event is already known.
+             *
+             * @param Key         - MIDI key number of the key where pressure changed
+             * @param Value       - MIDI key pressure value (0..127)
+             * @param MidiChannel - MIDI channel on which event occured on
+             *                      (low level indexing, means 0..15)
+             * @param FragmentPos - event's sample point position in the
+             *                      current audio fragment
+             * @see DispatchChannelPressure()
+             */
+            void DispatchPolyphonicKeyPressure(uint8_t Key, uint8_t Value, uint MidiChannel, int32_t FragmentPos);
+
             /**
              * Should be called by the implementing MIDI input device
              * whenever a program change event arrived. In case the
@@ -297,8 +392,48 @@ namespace LinuxSampler {
              */
             void DispatchProgramChange(uint8_t Program, uint MidiChannel);
 
+            /**
+             * Should be called by the implementing MIDI input device whenever
+             * a MIDI bank select MSB (Most Significant Byte) event arrived.
+             *
+             * In case the respective sampler channel(s) are enabled for "MIDI
+             * instrument mapping", a subsequent MIDI program change event can
+             * be used to let the respective sampler engine load another
+             * instrument according to the respective entry in the MIDI
+             * instrument map.
+             *
+             * MIDI sources can either a) just send bank select MSB events, or
+             * b) just send bank select LSB events or c) they can send both bank
+             * select MSB and LSB events. The sampler will automatically detect
+             * which style the MIDI source is using and appropriately interpret
+             * it for selecting the appropriate bank.
+             *
+             * @param BankMsb     - Most Significant Byte of the bank number
+             * @param MidiChannel - MIDI channel on which this bank select
+             *                      occurred
+             */
             void DispatchBankSelectMsb(uint8_t BankMsb, uint MidiChannel);
 
+            /**
+             * Should be called by the implementing MIDI input device whenever
+             * a MIDI bank select LSB (Least Significant Byte) event arrived.
+             *
+             * In case the respective sampler channel(s) are enabled for "MIDI
+             * instrument mapping", a subsequent MIDI program change event can
+             * be used to let the respective sampler engine load another
+             * instrument according to the respective entry in the MIDI
+             * instrument map.
+             *
+             * MIDI sources can either a) just send bank select MSB events, or
+             * b) just send bank select LSB events or c) they can send both bank
+             * select MSB and LSB events. The sampler will automatically detect
+             * which style the MIDI source is using and appropriately interpret
+             * it for selecting the appropriate bank.
+             *
+             * @param BankMsb     - Most Significant Byte of the bank number
+             * @param MidiChannel - MIDI channel on which this bank select
+             *                      occurred
+             */
             void DispatchBankSelectLsb(uint8_t BankLsb, uint MidiChannel);
 
             /**
@@ -315,7 +450,7 @@ namespace LinuxSampler {
              * Helper function for MIDI input devices that have the
              * MIDI data as raw bytes.
              *
-             * @param pData       - pointer to the raw MIDI data
+             * @param pData - pointer to the raw MIDI data
              */
             void DispatchRaw(uint8_t* pData);
 
@@ -341,6 +476,10 @@ namespace LinuxSampler {
             SynchronizedConfig<std::vector<VirtualMidiDevice*> > virtualMidiDevices;
             SynchronizedConfig<std::vector<VirtualMidiDevice*> >::Reader virtualMidiDevicesReader;
             Mutex virtualMidiDevicesMutex;
+            SynchronizedConfig<std::vector<uint8_t> > noteOnVelocityFilter;
+            SynchronizedConfig<std::vector<uint8_t> >::Reader noteOnVelocityFilterReader;
+            Mutex noteOnVelocityFilterMutex;
+            uint8_t runningStatusBuf[3];
 
             /**
              * Constructor
@@ -351,11 +490,31 @@ namespace LinuxSampler {
              * Destructor
              */
             virtual ~MidiInputPort();
+            
+            /**
+             * Takes a MIDI status byte (the first byte of each MIDI event) as
+             * argument and returns the expected size of the associated MIDI
+             * event according to the MIDI protocol. Returns -1 on invalid
+             * status bytes AND on variable size events (SysEx events).
+             *
+             * This method can be used for drivers which have to deal with raw
+             * MIDI data, like the CoreMIDI driver, which can receive MIDI
+             * packets with more than one event per packet.
+             *
+             * This method handles "MIDI running status" as well. That is, in
+             * case the supplied byte is not a status byte but a data byte,
+             * it expects the event to be in "running status" and accordingly
+             * uses the status byte of the previous event (processed by the
+             * Dispatch*() methods).
+             */
+            int expectedEventSize(unsigned char byte);
 
             friend class MidiInputDevice;
 
-        private:
+        private:            
             static SynchronizedConfig<std::set<Engine*> > SysexListeners; ///< All engines that are listening to sysex messages.
+            
+            uint8_t* handleRunningStatus(uint8_t* pData);
     };
 
 } // namsepace LinuxSampler
