@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005 - 2009 Christian Schoenebeck                       *
+ *   Copyright (C) 2005 - 2025 Christian Schoenebeck                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -41,12 +41,19 @@ namespace LinuxSampler {
         uint8_t uiMidiProgram;
         uint8_t uiMidiBankMsb;
         uint8_t uiMidiBankLsb;
-        uint8_t uiMidiRpnMsb; ///< MIDI Registered Parameter Number (upper 8 bits / coarse)
-        uint8_t uiMidiRpnLsb; ///< MIDI Registered Parameter Number (lower 8 bits / fine)
+        uint8_t uiMidiRpnMsb; ///< MIDI Registered Parameter Number (upper 7 bits / coarse)
+        uint8_t uiMidiRpnLsb; ///< MIDI Registered Parameter Number (lower 7 bits / fine)
+        uint8_t uiMidiRpnDataMsb; ///< New MIDI RPN Parameter Value (upper 7 bits / coarse)
+        uint8_t uiMidiRpnDataLsb; ///< New MIDI RPN Parameter Value (lower 7 bits / fine)
+        uint8_t uiMidiNrpnMsb; ///< MIDI Non-Registered Parameter Number (upper 7 bits / coarse)
+        uint8_t uiMidiNrpnLsb; ///< MIDI Non-Registered Parameter Number (lower 7 bits / fine)
+        uint8_t uiMidiNrpnDataMsb; ///< New MIDI NRPN Parameter Value (upper 7 bits / coarse)
+        uint8_t uiMidiNrpnDataLsb; ///< New MIDI NRPN Parameter Value (lower 7 bits / fine)
         bool    bMidiBankMsbReceived;
         bool    bMidiBankLsbReceived;
         bool    bProgramChangeReceived;
         bool    bMidiRpnReceived;
+        bool    bMidiNrpnReceived;
         int     iMidiInstrumentMap;
         atomic_t voiceCount;
         atomic_t diskStreamCount;
@@ -67,14 +74,31 @@ namespace LinuxSampler {
         SetVoiceCount(0);
         SetDiskStreamCount(0);
         p->pSamplerChannel = NULL;
-        ResetMidiRpnController();
+        ResetMidiRpnParameter();
+        ResetMidiNrpnParameter();
     }
 
     EngineChannel::~EngineChannel() {
         delete p;
     }
+    
+    /**
+     * Sometimes an instrument is splitted over several files. For example
+     * the GigaStudio format may split an instrument over a certain amount
+     * of files like: "Foo.gig", "Foo.gx01", "Foo.gx02", ...
+     * This method allows to retrieve the whole list of files that is used
+     * for the currently loaded instrument on this engine channel.
+     * Calling this method with index 0 is equivalent as calling the equal
+     * named method without any argument.
+     *
+     * @param index - index of sought file name (0, 1, 2, ...)
+     * @returns file name or empty string if index out of bounds
+     */
+    String EngineChannel::InstrumentFileName(int index) {
+        return (index == 0) ? InstrumentFileName() : "";
+    }
 
-    void EngineChannel::SetMute(int state) throw (Exception) {
+    void EngineChannel::SetMute(int state) LS_THROWS(Exception) {
         if (p->iMute == state) return;
         if (state < -1 || state > 1)
             throw Exception("Invalid Mute state: " + ToString(state));
@@ -164,7 +188,7 @@ namespace LinuxSampler {
         return (p->iMidiInstrumentMap == DEFAULT_MIDI_INSTRUMENT_MAP);
     }
 
-    int EngineChannel::GetMidiInstrumentMap() throw (Exception) {
+    int EngineChannel::GetMidiInstrumentMap() LS_THROWS(Exception) {
         if (UsesNoMidiInstrumentMap())
             throw Exception("EngineChannel is using no MIDI instrument map");
         if (UsesDefaultMidiInstrumentMap())
@@ -191,7 +215,7 @@ namespace LinuxSampler {
         StatusChanged(true);
     }
 
-    void EngineChannel::SetMidiInstrumentMap(int MidiMap) throw (Exception) {
+    void EngineChannel::SetMidiInstrumentMap(int MidiMap) LS_THROWS(Exception) {
         if (p->iMidiInstrumentMap == MidiMap) return;
 
         // check if given map actually exists in the MIDI instrument mapper
@@ -202,26 +226,126 @@ namespace LinuxSampler {
         StatusChanged(true);
     }
 
-    void EngineChannel::SetMidiRpnControllerMsb(uint8_t CtrlMSB) {
-        p->uiMidiRpnMsb = CtrlMSB;
+    // RPNs ...
+
+    void EngineChannel::SetMidiRpnParameterMsb(uint8_t ParamMSB) {
+        p->uiMidiRpnMsb = ParamMSB & 127;
         p->bMidiRpnReceived = true;
     }
 
-    void EngineChannel::SetMidiRpnControllerLsb(uint8_t CtrlLSB) {
-        p->uiMidiRpnLsb = CtrlLSB;
+    void EngineChannel::SetMidiRpnControllerMsb(uint8_t CtrlMSB) { // deprecated API
+        SetMidiRpnParameterMsb(CtrlMSB);
+    }
+
+    void EngineChannel::SetMidiRpnParameterLsb(uint8_t ParamLSB) {
+        p->uiMidiRpnLsb = ParamLSB & 127;
         p->bMidiRpnReceived = true;
     }
 
-    void EngineChannel::ResetMidiRpnController() {
+    void EngineChannel::SetMidiRpnControllerLsb(uint8_t CtrlLSB) { // deprecated API
+        SetMidiRpnParameterLsb(CtrlLSB);
+    }
+
+    void EngineChannel::SetMidiRpnDataMsb(uint8_t DataMSB) {
+        p->uiMidiRpnDataMsb = DataMSB & 127;
+    }
+
+    void EngineChannel::SetMidiRpnDataLsb(uint8_t DataLSB) {
+        p->uiMidiRpnDataLsb = DataLSB & 127;
+    }
+
+    void EngineChannel::SetMidiRpnData(int Data) {
+        if (Data < 0) Data = 0;
+        if (Data > 16383) Data = 16383;
+        p->uiMidiRpnDataMsb = (Data >> 7) & 127;
+        p->uiMidiRpnDataLsb = Data & 127;
+    }
+
+    void EngineChannel::ResetMidiRpnParameter() {
         p->uiMidiRpnMsb = p->uiMidiRpnLsb = 0;
+        p->uiMidiRpnDataMsb = p->uiMidiRpnDataLsb = 0;
         p->bMidiRpnReceived = false;
     }
 
-    int EngineChannel::GetMidiRpnController() {
+    void EngineChannel::ResetMidiRpnController() { // deprecated API
+        ResetMidiRpnParameter();
+    }
+
+    int EngineChannel::GetMidiRpnParameter() {
+        return (p->bMidiRpnReceived) ?
+               (p->uiMidiRpnMsb << 7) | p->uiMidiRpnLsb : -1;
+    }
+
+    int EngineChannel::GetMidiRpnController() { // deprecated API
         return (p->bMidiRpnReceived) ?
                (p->uiMidiRpnMsb << 8) | p->uiMidiRpnLsb : -1;
     }
 
+    int EngineChannel::GetMidiRpnData() {
+        return (p->bMidiRpnReceived) ?
+               (p->uiMidiRpnDataMsb << 7) | p->uiMidiRpnDataLsb : 0;
+    }
+
+    // NRPNs ...
+
+    void EngineChannel::SetMidiNrpnParameterMsb(uint8_t ParamMSB) {
+        p->uiMidiNrpnMsb = ParamMSB & 127;
+        p->bMidiNrpnReceived = true;
+    }
+
+    void EngineChannel::SetMidiNrpnControllerMsb(uint8_t CtrlMSB) { // deprecated API
+        SetMidiNrpnParameterMsb(CtrlMSB);
+    }
+
+    void EngineChannel::SetMidiNrpnParameterLsb(uint8_t ParamLSB) {
+        p->uiMidiNrpnLsb = ParamLSB & 127;
+        p->bMidiNrpnReceived = true;
+    }
+
+    void EngineChannel::SetMidiNrpnControllerLsb(uint8_t CtrlLSB) { // deprecated API
+        SetMidiNrpnParameterLsb(CtrlLSB);
+    }
+
+    void EngineChannel::SetMidiNrpnDataMsb(uint8_t DataMSB) {
+        p->uiMidiNrpnDataMsb = DataMSB & 127;
+    }
+
+    void EngineChannel::SetMidiNrpnDataLsb(uint8_t DataLSB) {
+        p->uiMidiNrpnDataLsb = DataLSB & 127;
+    }
+
+    void EngineChannel::SetMidiNrpnData(int Data) {
+        if (Data < 0) Data = 0;
+        if (Data > 16383) Data = 16383;
+        p->uiMidiNrpnDataMsb = (Data >> 7) & 127;
+        p->uiMidiNrpnDataLsb = Data & 127;
+    }
+
+    void EngineChannel::ResetMidiNrpnParameter() {
+        p->uiMidiNrpnMsb = p->uiMidiNrpnLsb = 0;
+        p->uiMidiNrpnDataMsb = p->uiMidiNrpnDataLsb = 0;
+        p->bMidiNrpnReceived = false;
+    }
+
+    void EngineChannel::ResetMidiNrpnController() { // deprecated API
+        ResetMidiNrpnParameter();
+    }
+
+    int EngineChannel::GetMidiNrpnParameter() {
+        return (p->bMidiNrpnReceived) ?
+               (p->uiMidiNrpnMsb << 7)  | p->uiMidiNrpnLsb : -1;
+    }
+
+    int EngineChannel::GetMidiNrpnController() { // deprecated API
+        return (p->bMidiNrpnReceived) ?
+               (p->uiMidiNrpnMsb << 8) | p->uiMidiNrpnLsb : -1;
+    }
+
+    int EngineChannel::GetMidiNrpnData() {
+        return (p->bMidiNrpnReceived) ?
+               (p->uiMidiNrpnDataMsb << 7) | p->uiMidiNrpnDataLsb : 0;
+    }
+
     uint EngineChannel::GetVoiceCount() {
         return atomic_read(&p->voiceCount);
     }
@@ -272,12 +396,15 @@ namespace LinuxSampler {
         }
     }
 
-    void EngineChannel::ExecuteProgramChange(uint8_t Program) {
-        dmsg(1,("Received MIDI program change (prog=%d)\n",Program));
+    void EngineChannel::ExecuteProgramChange(uint32_t Program) {
+        uint8_t hb = (Program >> 16) & 0xff;
+        uint8_t lb = (Program >> 8) & 0xff;
+        uint8_t pc = Program & 0x7f;
+
+        dmsg(1,("Received MIDI program change (msb=%d) (lsb=%d) (prog=%d)\n", hb ,lb, pc));
         std::vector<int> maps = MidiInstrumentMapper::Maps();
         if (maps.empty()) return;
 
-        SetMidiProgram(Program);
         if (UsesNoMidiInstrumentMap()) return;
         if (MidiInstrumentMapper::GetMapCount() == 0) return;
         // retrieve the MIDI instrument map this engine channel is assigned to
@@ -285,9 +412,9 @@ namespace LinuxSampler {
             ? MidiInstrumentMapper::GetDefaultMap() /*default*/ : GetMidiInstrumentMap();
         // is there an entry for this MIDI bank&prog pair in that map?
         midi_prog_index_t midiIndex;
-        midiIndex.midi_bank_msb = GetMidiBankMsb();
-        midiIndex.midi_bank_lsb = GetMidiBankLsb();
-        midiIndex.midi_prog     = GetMidiProgram();
+        midiIndex.midi_bank_msb = hb;
+        midiIndex.midi_bank_lsb = lb;
+        midiIndex.midi_prog     = pc;
         optional<MidiInstrumentMapper::entry_t> mapping =
             MidiInstrumentMapper::GetEntry(iMapID, midiIndex);
         if (mapping) { // if mapping exists ...
