@@ -3,7 +3,7 @@
  *   LinuxSampler - modular, streaming capable sampler                     *
  *                                                                         *
  *   Copyright (C) 2003, 2004 by Benno Senoner and Christian Schoenebeck   *
- *   Copyright (C) 2005 - 2016 Christian Schoenebeck                       *
+ *   Copyright (C) 2005 - 2025 Christian Schoenebeck                       *
  *                                                                         *
  *   This library is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -33,7 +33,7 @@
 
 namespace LinuxSampler {
 
-    FxSend::FxSend(EngineChannel* pEngineChannel, uint8_t MidiCtrl, String Name) throw (Exception)
+    FxSend::FxSend(EngineChannel* pEngineChannel, uint8_t MidiCtrl, String Name) LS_THROWS(Exception)
         : iDestinationEffectChain(-1), iDestinationEffectChainPos(-1), bInfoChanged(false)
     {
         this->pEngineChannel = pEngineChannel;
@@ -86,7 +86,7 @@ namespace LinuxSampler {
         return iDestinationEffectChainPos;
     }
 
-    void FxSend::SetDestinationEffect(int iChain, int iChainPos) throw (Exception) {
+    void FxSend::SetDestinationEffect(int iChain, int iChainPos) LS_THROWS(Exception) {
         AudioOutputDevice* pDevice = pEngineChannel->GetAudioOutputDevice();
         bool chainFound = false;
         if (iChain != -1) {
@@ -117,7 +117,7 @@ namespace LinuxSampler {
     }
 
     // TODO: to be removed
-    void FxSend::SetDestinationMasterEffect(int iChain, int iChainPos) throw (Exception) {
+    void FxSend::SetDestinationMasterEffect(int iChain, int iChainPos) LS_THROWS(Exception) {
         SetDestinationEffect(iChain, iChainPos);
     }
 
@@ -126,7 +126,7 @@ namespace LinuxSampler {
         return Routing[SrcChan];
     }
 
-    void FxSend::SetDestinationChannel(int SrcChan, int DstChan) throw (Exception) {
+    void FxSend::SetDestinationChannel(int SrcChan, int DstChan) LS_THROWS(Exception) {
         if (SrcChan < 0 || SrcChan >= pEngineChannel->Channels())
             throw Exception("Cannot alter FxSend routing, source channel out of bounds");
         AudioOutputDevice* pDevice = pEngineChannel->GetAudioOutputDevice();
@@ -182,7 +182,7 @@ namespace LinuxSampler {
         return MidiFxSendController;
     }
 
-    void FxSend::SetMidiController(uint8_t MidiCtrl) throw (Exception) {
+    void FxSend::SetMidiController(uint8_t MidiCtrl) LS_THROWS(Exception) {
         if (MidiCtrl >> 7)
             throw Exception("Invalid MIDI controller " + ToString((int)MidiCtrl));
         MidiFxSendController = MidiCtrl;
