@@ -22,5 +22,5 @@ int main( int argc, char **argv) {
     CppUnit::TestFactoryRegistry &registry = CppUnit::TestFactoryRegistry::getRegistry();
     runner.addTest( registry.makeTest() );
     bool wasSuccessful = runner.run( "", false );
-    return wasSuccessful;
+    return wasSuccessful ? 0 : 1;
 }
